/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 14 janvier 2007
   derniere modification le 10/07/2007 */

#ifndef __SCECTEXTURE_H__
#define __SCECTEXTURE_H__



/* donnees d'une texture 2D */
/* ----revise le 02/07/2007---- (faux) */
typedef struct sce_ctexdata SCE_CTexData;
struct sce_ctexdata
{
    void *texels;           /* donnees */
    
    /*int free_data;*/          /* booleen, true: les donnees devront etre
                               supprimees automatiquement */
    
    SCEenum target;         /* type des donnees (cube map uniquement) */
    
    int level;              /* niveau de mipmap auquel
                               correspondent les donnees */
    
    int pxsize;             /* taille d'un pixel en bytes */
    GLsizei size;           /* taille totale des donnees */
    GLint w, h, d;          /* dimensions */
    
    int comp;               /* booleen, true: les donnees de l'image sont
                                              stockees dans un format de pixel
                                              compresse */
    
    GLint internal;         /* format de pixel */
    GLenum format;          /* format de texture (type de texture) */
    GLenum type;            /* type des variables des donnees */
};


typedef struct sce_ctexture SCE_CTexture;
struct sce_ctexture
{
    SCEuint id;             /* identifiant opengl */
    
    SCEenum target;         /* type de la texture */
    
    SCEint base_level;      /* niveaux de mipmap */
    SCEint max_level;
    
    SCEint min_filter;      /* niveaux de filtrage */
    SCEint mag_filter;      
    
    int have_data;          /* booleen, true: des donnees ont etees envoyees */
    
    int tex1d, tex2d;       /* booleens parlant d'eux meme */
    int tex3d, texcub;
    
    int spec_mipmap;        /* niveaux de mipmap specifies */
    int use_mipmap;         /* generation automatique de mipmap demandee */
    int hw_mipmap;          /* generation automatique en hardware
                               de mipmap demande */
    
    int builded;            /* booleen, true: texture construite */
    
    unsigned int n_datas;   /* nombre de 'donnees' */
    SCE_CTexData *data;     /* donnees.
                               tableau representant les differents niveaux
                               de mipmap envoyes par l'utilisateur */
};



/* alloue de la memoire pour une texture */
SCE_CTexture* SCE_CCreateTexture(SCEenum);

/* supprime une texture */
void SCE_CDeleteTexture(SCE_CTexture*);

/* applique un target a une texture, en prenant soin de modifier egalement
   les booleens tex1d, tex2d, etc... */
int SCE_CSetTextureTarget(SCE_CTexture*, SCEenum);


/* affecte la structure de donnees numero id de la texture a data (memcpy) */
int SCE_CCopyTextureData(SCE_CTexture*, unsigned int, SCE_CTexData*);

/* associe les options des donnees pour coller avec la texture specifiee */
int SCE_CAssociateTextureData(SCE_CTexture*, SCE_CTexData*);

/* envoie des donnees a la texture (cree une nouvelle structure de donnees) */
int SCE_CAddTextureData(SCE_CTexture*, SCE_CTexData*);

/* renvoie l'ID de la donnee correspondante aux infos envoyees */
int SCE_CLocateTextureDataID(SCE_CTexture*, SCEenum, int);

/* met a jour les donnees specifiees
   (necessitera un UpdateTexture pour etre prit en comtpe) */
int SCE_CUpdateTextureData(SCE_CTexture*, int, SCE_CTexData*);

/* detruit toutes les donnees de la texture */
void SCE_CDeleteTextureData(SCE_CTexture*);



/* installe un filtre de redimensionnement a une texture */
void SCE_CSetTextureFilter(SCE_CTexture*, SCEint);

/* envoie un parametre specifie par l'utilisateur */
void SCE_CTextureParam(SCEenum, SCEint);
void SCE_CTextureParamf(SCEenum, SCEfloat);

/* generation des coordonnees de texture */
void SCE_CTextureGen(SCEenum, SCEenum, int);
void SCE_CTextureGenf(SCEenum, SCEenum, float);
void SCE_CTextureGenfv(SCEenum, SCEenum, float*);


/* construit la texture specifiee */
int SCE_CBuildTexture(SCE_CTexture*, int, int);


/* met a jour les donnees par defaut de la texture specifiee */
int SCE_CUpdateTexture(SCE_CTexture*);


/* specifie la texture active et son unite de texture */
void SCE_CUseTexture(unsigned int, SCE_CTexture*, unsigned int);


#endif /* guard */
