/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 21 decembre 2006
   derniere modification le 27/06/2007 */


#include <SCE/utils/SCEMath.h>
#include <SCE/utils/SCEVector.h>

#include <SCE/utils/SCEMatrix.h>


void SCE_Matrix4_LoadIdentity(SCE_TMatrix4 m)
{
    m[0] = m[5] = m[10] = m[15] = 1.0f;
    m[1] = m[2] = m[3] = m[4] =
    m[6] = m[7] = m[8] = m[9] =
    m[11] = m[12] = m[13] = m[14] = 0.0f;
}

void SCE_Matrix4_Mul(SCE_TMatrix4 n, SCE_TMatrix4 m)
{
    SCE_TMatrix4 r;
    
    r[0] = (m[0]*n[0]) + (m[1]*n[4]) + (m[2]*n[8]) + (m[3]*n[12]);
    r[4] = (m[4]*n[0]) + (m[5]*n[4]) + (m[6]*n[8]) + (m[7]*n[12]);
    r[8] = (m[8]*n[0]) + (m[9]*n[4]) + (m[10]*n[8]) + (m[11]*n[12]);
    r[12] = (m[12]*n[0]) + (m[13]*n[4]) + (m[14]*n[8]) + (m[15]*n[12]);
    
    r[1] = (m[0]*n[1]) + (m[1]*n[5]) + (m[2]*n[9]) + (m[3]*n[13]);
    r[5] = (m[4]*n[1]) + (m[5]*n[5]) + (m[6]*n[9]) + (m[7]*n[13]);
    r[9] = (m[8]*n[1]) + (m[9]*n[5]) + (m[10]*n[9]) + (m[11]*n[13]);
    r[13] = (m[12]*n[1]) + (m[13]*n[5]) + (m[14]*n[9]) + (m[15]*n[13]);
    
    r[2] = (m[0]*n[2]) + (m[1]*n[6]) + (m[2]*n[10]) + (m[3]*n[14]);
    r[6] = (m[4]*n[2]) + (m[5]*n[6]) + (m[6]*n[10]) + (m[7]*n[14]);
    r[10] = (m[8]*n[2]) + (m[9]*n[6]) + (m[10]*n[10]) + (m[11]*n[14]);
    r[14] = (m[12]*n[2]) + (m[13]*n[6]) + (m[14]*n[10]) + (m[15]*n[14]);
    
    r[3] = (m[0]*n[3]) + (m[1]*n[7]) + (m[2]*n[11]) + (m[3]*n[15]);
    r[7] = (m[4]*n[3]) + (m[5]*n[7]) + (m[6]*n[11]) + (m[7]*n[15]);
    r[11] = (m[8]*n[3]) + (m[9]*n[7]) + (m[10]*n[11]) + (m[11]*n[15]);
    r[15] = (m[12]*n[3]) + (m[13]*n[7]) + (m[14]*n[11]) + (m[15]*n[15]);
    
    n[0] = r[0]; n[1] = r[1]; n[2] = r[2]; n[3] = r[3];
    n[4] = r[4]; n[5] = r[5]; n[6] = r[6]; n[7] = r[7];
    n[8] = r[8]; n[9] = r[9]; n[10] = r[10]; n[11] = r[11];
    n[12] = r[12]; n[13] = r[13]; n[14] = r[14]; n[15] = r[15];
}


void SCE_Matrix4_Add(SCE_TMatrix4 a, SCE_TMatrix4 b)
{
    a[0] += b[0]; a[1] += b[1]; a[2] += b[2]; a[3] += b[3];
    a[4] += b[4]; a[5] += b[5]; a[6] += b[6]; a[7] += b[7];
    a[8] += b[8]; a[9] += b[9]; a[10] += b[10]; a[11] += b[11];
    a[12] += b[12]; a[13] += b[13]; a[14] += b[14]; a[15] += b[15];
}

void SCE_Matrix4_Sub(SCE_TMatrix4 a, SCE_TMatrix4 b)
{
    a[0] -= b[0]; a[1] -= b[1]; a[2] -= b[2]; a[3] -= b[3];
    a[4] -= b[4]; a[5] -= b[5]; a[6] -= b[6]; a[7] -= b[7];
    a[8] -= b[8]; a[9] -= b[9]; a[10] -= b[10]; a[11] -= b[11];
    a[12] -= b[12]; a[13] -= b[13]; a[14] -= b[14]; a[15] -= b[15];
}


void SCE_Matrix4_Transpose(SCE_TMatrix4 m)
{
    float tmp;
    
    tmp = m[1]; m[1] = m[4]; m[4] = tmp;
    tmp = m[2]; m[2] = m[8]; m[8] = tmp;
    tmp = m[3]; m[3] = m[12]; m[12] = tmp;
    tmp = m[6]; m[6] = m[9]; m[9] = tmp;
    tmp = m[7]; m[7] = m[13]; m[13] = tmp;
    tmp = m[11]; m[11] = m[14]; m[14] = tmp;
}

void SCE_Matrix4_RotX(SCE_TMatrix4 m, float a)
{
    float cosinus, sinus;
    
    cosinus = cos(a);
    sinus = sin(a);
    
    m[5] = cosinus;
    m[6] = -sinus;
    m[9] = sinus;
    m[10] = cosinus;
}

void SCE_Matrix4_RotY(SCE_TMatrix4 m, float a)
{
    float cosinus, sinus;
    
    cosinus = cos(a);
    sinus = sin(a);
    
    m[0] = cosinus;
    m[2] = -sinus;
    m[8] = sinus;
    m[10] = cosinus;
}

void SCE_Matrix4_RotZ(SCE_TMatrix4 m, float a)
{
    float cosinus, sinus;
    
    cosinus = cos(a);
    sinus = sin(a);
    
    m[0] = cosinus;
    m[1] = -sinus;
    m[4] = sinus;
    m[5] = cosinus;
}


/* revise le 23/06/2007 */
void SCE_Matrix4_Rotate(SCE_TMatrix4 m, float a, float x, float y, float z)
{
    float val[9];
    
    #define COSINUS 0
    #define SINUS 1
    #define UN_COS 2
    #define XSIN 3
    #define YSIN 4
    #define ZSIN 5
    #define XY 6
    #define XZ 7
    #define YZ 8
    
    val[COSINUS] = cos(a);
    val[SINUS] = sin(a);
    
    val[UN_COS] = 1.0f - val[COSINUS];
    
    val[XSIN] = x*val[SINUS];
    val[YSIN] = y*val[SINUS];
    val[ZSIN] = z*val[SINUS];
    
    val[XY] = x*y;
    val[XZ] = x*z;
    val[YZ] = y*z;
    
    m[0] = (x*x*val[UN_COS]) + val[COSINUS];
    m[1] = (val[XY]*val[UN_COS]) - val[ZSIN];
    m[2] = (val[XZ]*val[UN_COS]) + val[YSIN];
    m[3] = 0.0f;
    
    m[4] = (val[XY]*val[UN_COS]) + val[ZSIN];
    m[5] = (y*y*val[UN_COS]) + val[COSINUS];
    m[6] = (val[YZ]*val[UN_COS]) - val[XSIN];
    m[7] = 0.0f;
    
    m[8] = (val[XZ]*val[UN_COS]) - val[YSIN];
    m[9] = (val[YZ]*val[UN_COS]) + val[XSIN];
    m[10] = (z*z*val[UN_COS]) + val[COSINUS];
    m[11] = 0.0f;
    
    m[12] = m[13] = m[14] = 0.0f;
    m[15] = 1.0f;
    
    #undef COSINUS
    #undef SINUS
    #undef UN_COS
    #undef XSIN
    #undef YSIN
    #undef ZSIN
    #undef XY
    #undef XZ
    #undef YZ
}


/* ajoute le 23/06/2007 */
void SCE_Matrix4_Rotatev(SCE_TMatrix4 m, float a, SCE_TVector3 axe)
{
    SCE_Matrix4_Rotate(m, a, axe[0], axe[1], axe[2]);
}


void SCE_Matrix4_MulV3(SCE_TMatrix4 m, SCE_TVector3 v)
{
    SCE_TVector3 r;
    
    r[0] = m[0]*v[0] + m[1]*v[1] + m[2]*v[2] + m[3];
    r[1] = m[4]*v[0] + m[5]*v[1] + m[6]*v[2] + m[7];
    r[2] = m[8]*v[0] + m[9]*v[1] + m[10]*v[2] + m[11];
    
    v[0] = r[0];
    v[1] = r[1];
    v[2] = r[2];
}
